<template>
  <AdminLayout>
    <DashboardHeader>
      <DashboardTitle>{{ $t("app.links.title") }}</DashboardTitle>
    </DashboardHeader>
    <DashboardBody>
      <ListActions>
        <FiltersSearch v-model="filters.search" />
        <ButtonGroup>
          <FiltersSelect
            v-model="filters"
            hide-button
            @clear="filters.clear"
            @updated="fetchLinks"
          />
        </ButtonGroup>
      </ListActions>
      <template v-if="linksFetch.hasErrors">
        <ErrorState />
      </template>
      <template v-else>
        <ListGroup :is-loading="linksFetch.isFetching">
          <ListTable>
            <ListTableHeader>
              <ListTableRow>
                <ListTableCell primary>{{ $t("app.links.form.url") }}</ListTableCell>
                <ListTableCell>{{ $t("app.links.form.user") }}</ListTableCell>
                <ListTableCell>{{ $t("app.links.form.visits_count") }}</ListTableCell>
                <ListTableCell>{{ $t("app.links.form.password") }}</ListTableCell>
                <ListTableCell>{{ $t("app.links.form.expires_at") }}</ListTableCell>
                <ListTableCell>{{ $t("app.links.form.updated_at") }}</ListTableCell>
                <ListTableCell />
              </ListTableRow>
            </ListTableHeader>
            <ListTableBody>
              <ListTableRow
                v-for="link in collection.data"
                :key="link.id"
              >
                <ListTableCell>
                  <div class="flex items-center gap-2">
                    <img
                      :src="link.favicon"
                      alt="Favicon"
                      class="size-4"
                    />
                    <a
                      :href="link.url"
                      class="block max-w-40 truncate font-medium hover:underline sm:max-w-md 2xl:max-w-xl"
                      rel="noreferrer"
                      target="_blank"
                    >
                      {{ link.url }}
                    </a>
                  </div>
                  <button
                    class="text-muted inline-flex outline-none hover:underline"
                    type="button"
                    @click="copyToClipboard(link.short_url)"
                  >
                    {{ link.short_url }}
                  </button>
                </ListTableCell>
                <ListTableCell>
                  <template v-if="link.user">
                    <div class="flex items-center gap-2">
                      <UserAvatar
                        :user-name="link.user?.name"
                        class="inline-block size-7 rounded-full"
                      />
                      <div class="flex flex-col text-sm text-gray-800">
                        <div>{{ link.user?.name }}</div>
                        <div class="text-sm">
                          {{ link.user?.email }}
                        </div>
                      </div>
                    </div>
                  </template>
                  <template v-else>
                    <span>-</span>
                  </template>
                </ListTableCell>
                <ListTableCell>
                  {{ link.visits_count }}
                </ListTableCell>
                <ListTableCell>
                  <template v-if="link.has_password">
                    <CheckRegularIcon class="size-5 fill-green-500" />
                  </template>
                  <template v-else>
                    <XMarkRegularIcon class="size-5 fill-red-500" />
                  </template>
                </ListTableCell>
                <ListTableCell>
                  {{ link.expires_at ? new Date(link.expires_at).toLocaleDateString() : "-" }}
                </ListTableCell>
                <ListTableCell>
                  {{ new Date(link.updated_at).toLocaleDateString() }}
                </ListTableCell>
                <ListTableCell>
                  <LinksDropdown :link="link" />
                </ListTableCell>
              </ListTableRow>
            </ListTableBody>
          </ListTable>
          <ListEmpty
            v-if="collection.empty() && !linksFetch.isFirthFetch"
            :icon="LinkShortenerIllustration"
            description="app.links.empty.description"
            title="app.links.empty.title"
          />
        </ListGroup>
        <Pagination
          :collection="collection"
          @change-page="filters.changePage"
        />
      </template>
    </DashboardBody>
  </AdminLayout>
  <RouterView @fetch="fetchLinks()" />
</template>

<script lang="ts" setup>
import CheckRegularIcon from "@/svg/icons/check-regular.svg?component";
import XMarkRegularIcon from "@/svg/icons/xmark-regular.svg?component";
import DashboardHeader from "@/js/contexts/shared/ui/components/dashboard/DashboardHeader.vue";
import DashboardBody from "@/js/contexts/shared/ui/components/dashboard/DashboardBody.vue";
import DashboardTitle from "@/js/contexts/shared/ui/components/dashboard/DashboardTitle.vue";
import useFetch from "@/js/contexts/shared/composables/useFetch";
import useCollection from "@/js/contexts/shared/composables/useCollection";
import useFilters from "@/js/contexts/shared/composables/useFilters";
import type PaginatedCollection from "@/js/contexts/shared/models/PaginatedCollection";
import { pickBy } from "lodash-es";
import type { LinkWithUser } from "@/js/contexts/shared/models/Link";
import ButtonGroup from "@/js/contexts/shared/ui/components/button/ButtonGroup.vue";
import ListGroup from "@/js/contexts/shared/ui/components/list/ListGroup.vue";
import FiltersSelect from "@/js/contexts/shared/ui/components/filters/FiltersSelect.vue";
import FiltersSearch from "@/js/contexts/shared/ui/components/filters/FiltersSearch.vue";
import ListActions from "@/js/contexts/shared/ui/components/list/ListActions.vue";
import ErrorState from "@/js/contexts/shared/ui/components/error/ErrorState.vue";
import Pagination from "@/js/contexts/shared/ui/components/pagination/Pagination.vue";
import ListTable from "@/js/contexts/shared/ui/components/list/ListTable.vue";
import ListEmpty from "@/js/contexts/shared/ui/components/list/ListEmpty.vue";
import LinkShortenerIllustration from "@/svg/illustrations/link-shortener.svg?component";
import ListTableRow from "@/js/contexts/shared/ui/components/list/ListTableRow.vue";
import ListTableHeader from "@/js/contexts/shared/ui/components/list/ListTableHeader.vue";
import ListTableCell from "@/js/contexts/shared/ui/components/list/ListTableCell.vue";
import ListTableBody from "@/js/contexts/shared/ui/components/list/ListTableBody.vue";
import { useClipboard } from "@vueuse/core";
import useNotification from "@/js/contexts/shared/composables/useNotification";
import LinksDropdown from "@/js/contexts/admin/links/ui/components/LinksDropdown.vue";
import AdminLayout from "@/js/contexts/admin/shared/ui/layouts/AdminLayout.vue";
import UserAvatar from "@/js/contexts/shared/ui/components/avatar/UserAvatar.vue";

const { t } = useI18n();
const { successNotification } = useNotification();

useHead({
  title: t(`app.links.title`),
});

const linksFetch = useFetch("/api/admin/links");
const collection = useCollection<LinkWithUser>();
const filters = useFilters(
  {
    search: "",
  },
  ["search"],
);

onMounted(() => {
  fetchLinks();
});

const fetchLinks = () => {
  linksFetch.get<PaginatedCollection<LinkWithUser>>(pickBy(filters.value.data())).then((response) => {
    collection.fromResponse(response.data);
  });
};
const copyToClipboard = (value: string) => {
  useClipboard()
    .copy(value)
    .then(() => successNotification(t("app.clipboard.copied")));
};
</script>
